var ID = 'RTAdapter';

var non_text_format = ['js', 'png', 'jpg', 'bmp', 'jpeg', 'gif', 'ico', 'tiff', 'webp', 'image', 'mp3', 'ogg', 'wav', 'm4a', 'font', 'eot', 'ttf', 'woff', 'svg', 'ttc'];

var REGEX = /^\w+:\/\/.*/;

var fs = jsb.fileUtils;
var writablePath = fs.getWritablePath() + 'remote-asset/';

var downloader = null;
var downloadMap = new Map();

var RTAdapter = window.RTAdapter = function () {
    this.id = ID;
    this.async = true;
    this.pipeline = null;
    this.REMOTE_SERVER_ROOT = 'https://cg-cdn.leuok.com/cgtx';
    //this.REMOTE_SERVER_ROOT = 'http://192.168.2.71:3000/tt';
};
RTAdapter.ID = ID;

RTAdapter.prototype.handle = function (item, callback) {
    if (item.type === 'js') {
        callback(null, null);
        return;
    }
    if (item.type === 'uuid') {
        var result = cc.Pipeline.Downloader.PackDownloader.load(item, callback);
        // handled by PackDownloader
        if (result !== undefined) {
            // null result
            if (!!result) {
                return result;
            } else {
                return;
            }
        }
    }

    var filePath = item.url;
    // Read from package
    if (fs.isFileExist(filePath)) {
        if (item.type && non_text_format.indexOf(item.type) !== -1) {
            nextPipe(item, callback);
        } else {
            readText(item, callback);
        }
    } else {
        readFromLocal(item, callback);
    }
};

var rtAdapter = window.rtAdapter = new RTAdapter();
cc.loader.insertPipeAfter(cc.loader.assetLoader, rtAdapter);
console.log('insert rtAdapter finish');

function nextPipe(item, callback) {
    var queue = cc.LoadingItems.getQueue(item);
    queue.addListener(item.id, function (item) {
        if (item.error) {
            if (fs.removeFile(item.url)) {
                cc.log('Load failed, removed local file ' + item.url + ' successfully!');
            } else {
                cc.error('Load failed, removed local file ' + item.url + ' failed!');
            }
        }
    });
    callback(null, null);
}

function readText(item, callback) {
    var url = item.url;
    var res = jsb.fileUtils.getStringFromFile(url);
    if (res) {
        item.states[cc.loader.downloader.id] = cc.Pipeline.ItemState.COMPLETE;
        callback(null, res);
    } else {
        cc.warn('Read file failed: ' + url);
        if (fs.removeFile(url)) {
            cc.log('Read file failed, removed local file ' + url + ' successfully!');
        } else {
            cc.error('Read file failed, removed local file ' + url + ' failed!');
        }
        callback({
            status: 0,
            errorMessage: res && res.errMsg ? res.errMsg : "Read text file failed: " + url
        });
    }
}

function readFromLocal(item, callback) {
    var localPath = writablePath + item.url;
    // Read from local file cache
    if (fs.isFileExist(localPath)) {
        item.url = localPath;
        item.rawUrl = localPath;
        if (item.type && non_text_format.indexOf(item.type) !== -1) {
            nextPipe(item, callback);
        } else {
            readText(item, callback);
        }
    } else {
        // No remote server indicated, then continue to downloader
        if (!rtAdapter.REMOTE_SERVER_ROOT) {
            callback(null, null);
            return;
        }

        downloadRemoteFile(item, callback);
    }
}

function downloadRemoteFile(item, callback) {
    // Download from remote server
    var relatUrl = item.url;
    // filter protocol url (E.g: https:// or http:// or ftp://)
    if (REGEX.test(relatUrl)) {
        callback(null, null);
        return;
    }

    var remoteUrl = rtAdapter.REMOTE_SERVER_ROOT + '/' + relatUrl;
    item.url = remoteUrl;
    var localPath = writablePath + relatUrl;
    
    // console.log('downloadRemoteFile url:' + remoteUrl);

    if (!downloader) {
        downloader = new jsb.Downloader();
        //下载错误
        downloader.setOnTaskError(function (sender, errorCode, errorCodeInternal, errorStr) {
            console.log("on download task error:" + errorStr + ' url:' + sender.requestURL);
            var failedTask = downloadMap.get(sender.requestURL);
            if (failedTask) {
                downloadMap.delete(sender.requestURL);
                if (failedTask.fail) {
                    failedTask.fail();
                }
            }
        });
        //下载完成
        downloader.setOnFileTaskSuccess(function (sender) {
            // console.log("on download success url:" + sender.requestURL + ' path:' + sender.storagePath);
            var successTask = downloadMap.get(sender.requestURL);
            if (successTask) {
                downloadMap.delete(sender.requestURL);
                if (successTask.success) {
                    successTask.success();
                }
            }
        });
    }

    if (cc.sys.os === cc.sys.OS_IOS) {
        makeDirecotryRecursively(localPath);
    }

    var task = {
        url: remoteUrl,
        path: localPath,
        success: function () {
            item.url = localPath;
            if (item.type && non_text_format.indexOf(item.type) !== -1) {
                nextPipe(item, callback);
            } else {
                readText(item, callback);
            }
        },
        fail: function (errMsg) {
            callback({
                status: 0,
                errorMessage: errMsg ? errMsg : "Download file failed: " + remoteUrl
            });
        },
    };
    downloadMap.set(remoteUrl, task);
    downloader.createDownloadFileTask(remoteUrl, localPath, "");
}

function makeDirecotryRecursively (dir) {
    try {
        if (fs.isDirectoryExist(dir)) {
            return;
        } else {
            const index = dir.lastIndexOf('/');
            if (index >= 0) {
                const parent = dir.substring(0, index);
                if (parent.length > 0 && parent !== dir) {
                    if (!fs.isDirectoryExist(parent)) {
                        makeDirecotryRecursively(parent);
                    }
                    // console.log('createDirectory:' + dir);
                    fs.createDirectory(dir);
                }
            }
        }
    } catch (e) {
        console.error('makeDirecotryRecursively error, dir:' + dir);
    }
}